from eplasty.session import Session

class EPDataManager():
    
    def __init__(self, pool, connection ):
        self.connection = connection
        self.session = Session(self.connection)
        self.pool = pool

    def abort(self, transaction):
        pass

    def tpc_begin(self, transaction):
        pass

    def tpc_vote(self, transaction):
        pass

    def commit(self, transaction):
        self.session.flush()

    def tpc_finish(self, transaction):
        self.connection.commit()
        self.pool.putconn(self.connection)

    def tpc_abort(self, transaction):
        self.connection.rollback()
        self.pool.putconn(self.connection)

    abort = tpc_abort



import abc
from eplasty.object.exc import NotFound
import weakref

class Collection(metaclass=abc.ABCMeta):
    """Collection wrap around object classes and allow dictionary-like
    lookup"""

    def __init__(self, **kwargs):
        for kwarg in ['class_', 'field', 'request']:
            if kwarg in kwargs:
                setattr(self, kwarg, kwargs[kwarg])
        if 'request' in kwargs:
            self.session = kwargs['request'].ep_session
    
    def mount(self, parent, name):
        self.__parent__ = weakref.proxy(parent)
        self.__name__ = name
        parent[name] = self

    def get_all(self, **kwargs):
        result = list(self.class_.find(session=self.session, **kwargs))
        for item in result:
            item.__parent__ = self
        return result

    values = get_all


    def __getitem__(self, key):
        try:
            result = self.class_.get(
                getattr(self.class_, self.field) == key, session=self.session
            )
        except NotFound:
            raise KeyError(key)
        result.__parent__ = self
        if not (hasattr(result, '__name__') and result.__name__):
            result.__name__ = key
        return result



import eplasty as ep
import psycopg2.pool
import transaction

from pyramid_eplasty.data_manager import EPDataManager


def eplasty_tween_factory(handler, registry):
    config=registry.settings
    connection_pool = psycopg2.pool.ThreadedConnectionPool(
        5, 10, 
        host = config.get('eplasty.host', '127.0.0.1'),
        port = config.get('eplasty.port', '5432'),
        database = config['eplasty.database'],
        user = config['eplasty.username'],
        password = config['eplasty.passwd'],
        connection_factory=ep.cursor.EPConnection,
    )
    def eplasty_tween(request):
        conn = connection_pool.getconn()
        manager = EPDataManager(connection_pool, conn)
        request.ep_session = manager.session
        transaction.get().join(manager)
        return handler(request)
    return eplasty_tween

def includeme(config):
    config.add_tween(
        'pyramid_eplasty.tween.eplasty_tween_factory',
        under='pyramid_tm.tm_tween_factory'
    )



# vim set fileencoding=utf-8
from setuptools import setup

setup(
      name = 'pyramid_eplasty',
      version = '0.1.2',
      author = 'Szymon Pyżalski',
      author_email = 'zefciu <szymon@pythonista.net>',
      description =
        'Include this in a pyramid project to get elephantoplasty support',
      license = 'BSD',
      url = 'http://github.com/zefciu/Elephantoplasty',
      keywords = 'orm postgresql psql pg persistence sql relational database',
      
      install_requires = ['pyramid>=1.2', 'Elephantoplasty>=0.0.2'],
      packages = ['pyramid_eplasty'],
      classifiers = [
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Topic :: Database :: Front-Ends',
      ]
    
)
